import com.codahale.metrics.Slf4jReporter.LoggingLevel
import com.fasterxml.jackson.databind.ObjectMapper
import org.apache.spark.broadcast.Broadcast
import org.apache.spark.sql.SparkSession
import org.apache.spark.{SparkConf, SparkContext}

/**
  * Created by tord on 07.03.17.
  */

object TestCreateIndex {


  def main(args: Array[String]): Unit = {

    val appName: String = "Indexing with Spark"
    val inputPath: String = args.apply(0)
    val outputPath: String = args.apply(1)
    val index: Int = args.apply(2).toInt
    val analytics = if (args.isDefinedAt(3) && args.apply(3).toInt == 1) true else false
    val indexPrecision: Int = 12
    val queryPrecision: Int = if(args.isDefinedAt(4)) args.apply(4).toInt else 12
    val queryTerm = if(args.isDefinedAt(5)) args.apply(5) else "home"
    val indexPartitionPrecision: Int = 6

    println("Usage: ")
    println("<inputPath> <outputPath> <index(1/0)> <query(1/0)> <queryPrecision(1-12)> <queryTerm>")
    val conf = new SparkConf().setAppName(appName)
    conf.set("userOutputPath", outputPath +"/")
    conf.set("spark.eventLog.enabled", "true")
    conf.set("analytics", analytics.toString)
    conf.set("spark.sql.hive.metastorePartitionPruning", "true")
    conf.set("spark.sql.shuffle.partitions", "2048")

    val sc: SparkContext = new SparkContext(conf)
    val spark = SparkSession
      .builder()
      .getOrCreate()

    spark.sparkContext.setLogLevel("WARN")


    // Either indexes the dataset or queries
    if(index == 1){
      SpatioTextualIndex.build(spark, inputPath, indexPrecision, queryPrecision, indexPartitionPrecision)
    }else if (analytics){
      SpatioTextualIndex.fullQuery((40.754669,-73.986053), queryTerm, queryPrecision, spark)
    }
    spark.stop()
  }
}
